{
Copyright (c) 2011, Carl-Philip Hänsch

Permission to use, copy, modify, and/or distribute this software for any purpose with or
without fee is hereby granted, provided that the above copyright notice and this permission
notice appear in all copies and for each binary release, a linux version and the source code
is developed and released.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
OR PERFORMANCE OF THIS SOFTWARE.
}

// will man die ältere Version enet 1.2 nutzen, so muss man ENET12 definieren mit {$define ENET12}

unit enet;
{$mode objfpc}{$H+}
{$packrecords c}

interface

const
   ENET_PACKET_FLAG_RELIABLE    = (1 shl 0); // kommt immer an
   ENET_PACKET_FLAG_UNSEQUENCED = (1 shl 1); // Reihenfolge egal (nur unreliable)
   ENET_PACKET_FLAG_NO_ALLOCATE = (1 shl 2); // ??
   
   ENET_EVENT_TYPE_NONE       = 0;
   ENET_EVENT_TYPE_CONNECT    = 1;
   ENET_EVENT_TYPE_DISCONNECT = 2;
   ENET_EVENT_TYPE_RECEIVE    = 3;
   
   
   ENET_HOST_ANY       = 0;
   _ENET_HOST_BROADCAST = $FFFFFFFF;

   ENET_PORT_ANY       = 0;

  
type

     size_t = ptrint;
     
     PEnetListNode = ^ENetListNode;
     ENetListNode = record
       next: PEnetListNode;
       previous: PEnetListNode;
     end;

     PENetHost = ^ENetHost;
     PENetAddress = ^ENetAddress;
     PENetPeer = ^ENetPeer;
     PENetPacket = ^ENetPacket;
     PENetEvent = ^ENetEvent;

    
     ENetHost = record end;
     
     ENetAddress = record
      host: cardinal;
      port: word;
     end;

     ENetPeerState = byte;
     
     ENetPeer = record
        {$IFNDEF ENET12}dispatchList: EnetListNode;{$ENDIF}
        host : PENetHost;
        outgoingPeerID : word;
        incomingPeerID : word;
        {$IFNDEF ENET12}
        connectID: cardinal;
        outgoingSessionID: byte;
        incomingSessionID: byte;
        {$ELSE}
        sessionID : cardinal;
        {$ENDIF}
        address : ENetAddress;
        data : pointer;
        state : ENetPeerState;
        channels : pointer;
        channelCount : size_t;
        incomingBandwidth : cardinal;
        outgoingBandwidth : cardinal;
        incomingBandwidthThrottleEpoch : cardinal;
        outgoingBandwidthThrottleEpoch : cardinal;
        incomingDataTotal : cardinal;
        outgoingDataTotal : cardinal;
        lastSendTime : cardinal;
        lastReceiveTime : cardinal;
        nextTimeout : cardinal;
        earliestTimeout : cardinal;
        packetLossEpoch : cardinal;
        packetsSent : cardinal;
        packetsLost : cardinal;
        packetLoss : cardinal;
        packetLossVariance : cardinal;
        packetThrottle : cardinal;
        packetThrottleLimit : cardinal;
        packetThrottleCounter : cardinal;
        packetThrottleEpoch : cardinal;
        packetThrottleAcceleration : cardinal;
        packetThrottleDeceleration : cardinal;
        packetThrottleInterval : cardinal;
        lastRoundTripTime : cardinal;
        lowestRoundTripTime : cardinal;
        lastRoundTripTimeVariance : cardinal;
        highestRoundTripTimeVariance : cardinal;
        roundTripTime : cardinal;
        roundTripTimeVariance : cardinal;
        {$IFNDEF ENET12}
        mtu: cardinal;
        {$ELSE}
        mtu : word;
        {$ENDIF}
        windowSize : cardinal;
        reliableDataInTransit : cardinal;
        outgoingReliableSequenceNumber : word;
        (*acknowledgements : ENetList;
        sentReliableCommands : ENetList;
        sentUnreliableCommands : ENetList;
        outgoingReliableCommands : ENetList;
        outgoingUnreliableCommands : ENetList;
        {$IFNDEF ENET12}
        dispatchedCommands: ENetList;
        needsDispatch: integer;
        {$ENDIF}
        incomingUnsequencedGroup : enet_uint16;
        outgoingUnsequencedGroup : enet_uint16;
        unsequencedWindow : array[0..(ENET_PEER_UNSEQUENCED_WINDOW_SIZE/32)-1] of enet_uint32;
        {$IFNDEF ENET12}
        eventData: cardinal;
        {$ELSE}
        disconnectData : cardinal;
        {$ENDIF}
        *)
      end;
   
   ENetEvent = record
     typ: byte;
     peer: PENetPeer;
     channel: byte;
     data: integer;
     packet: ^ENetPacket;
   end;
   
   ENetPacket = record
     referenceCount : size_t;
     flags : cardinal;
     data : pointer;
     dataLength : size_t;
     freeCallback : procedure(packet:  PENetPacket);
   end;
   

const prefix={$ifdef unix}''{$else}''{$endif};
      enet_lib='enet';

function enet_host_create(const adr: PENetAddress; connections: integer; {$IFNDEF ENET12}channelLimit: size_t;{$ENDIF} downstream: integer; upstream: integer): PENetHost; cdecl; external enet_lib name prefix+'enet_host_create';
procedure enet_host_destroy(const host: PENetHost); cdecl; external enet_lib name prefix+'enet_host_destroy';
function enet_host_connect(host: PENetHost; const adr: PENetAddress; {$IFNDEF ENET12}channelCount: size_t;{$ENDIF} data: integer): PENetPeer; cdecl; external enet_lib name prefix+'enet_host_connect';
function enet_host_service (host: PENetHost; event: PENetEvent; timeout: integer): integer; cdecl; external enet_lib name prefix+'enet_host_service';
procedure enet_host_flush(host: PENetHost); cdecl; external enet_lib name prefix+'enet_host_flush';
procedure enet_host_broadcast(host: PENetHost; channel: byte; packet: PENetPacket); cdecl; external enet_lib name prefix+'enet_host_broadcast';
procedure enet_host_broadcast_string(host: PENetHost; s: string; flags: cardinal);

function enet_address_set_host(address: PENetAddress; const hostName: PChar): integer; cdecl; external enet_lib name prefix+'enet_address_set_host';
function enet_address_string(const address: ENetAddress): string;

function enet_packet_create(data: pointer; size: size_t; flags: cardinal): PENetPacket; cdecl; external enet_lib name prefix+'enet_packet_create';
procedure enet_packet_destroy(packet: PENetPacket); cdecl; external enet_lib name prefix+'enet_packet_destroy';

function enet_peer_send(peer: PENetPeer; channel: byte; packet: PENetPacket): integer; cdecl; external enet_lib name prefix+'enet_peer_send';
procedure enet_peer_reset(peer: PENetPeer); cdecl; external enet_lib name prefix+'enet_peer_reset';
procedure enet_peer_disconnect(peer: PENetPeer; data: cardinal); cdecl; external enet_lib name prefix+'enet_peer_disconnect';

procedure enet_peer_send_string(peer: PENetPeer; s: string; flags: cardinal);

implementation
uses sysutils;

function enet_initialize: integer; cdecl; external enet_lib name prefix+'enet_initialize';
procedure enet_deinitialize; cdecl; external enet_lib name prefix+'enet_deinitialize';

procedure enet_host_broadcast_string(host: PENetHost; s: string; flags: cardinal);
var packet: PENetPacket;
begin
  packet:=enet_packet_create(@s[1], length(s), flags);
  enet_host_broadcast(host, 0, packet);
end;

function enet_address_string(const address: ENetAddress): string;
begin
  result:=IntToStr((address.host shr 00) and $FF)+'.'+IntToStr((address.host shr 08) and $FF)
     +'.'+IntToStr((address.host shr 16) and $FF)+'.'+IntToStr((address.host shr 24) and $FF)
     +':'+IntToStr(address.port);
end;

procedure enet_peer_send_string(peer: PENetPeer; s: string; flags: cardinal);
var packet: PENetPacket;
begin
  packet:=enet_packet_create(@s[1], length(s), flags);
  enet_peer_send(peer, 0, packet);
end;

initialization
  if (enet_initialize() <> 0) then begin
   writeln(stderr, 'Warning: Could not initialize enet lib');
  end;
finalization
  enet_deinitialize;

end.
