unit generalhelpers;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

const
      bst = ['a'..'z','A'..'Z','_','#'];
      leer = [' ',#9,#13];
      nummer = ['0'..'9'];
      hnummer = nummer+['A'..'F','a'..'f'];
      operators: array[0..17,0..1] of char=('==','+=','-=','*=','/=','\=','%=','|=','&=','!=','>=','<=',':=','<>','&&','||','++','--'); // zusammengesetzte Operatoren

type

{ TTreeComponent }

     // Threadsicherheit: Erstellen und Befreien muss hintereinander passieren
     // ansonsten muss diese Komponente threadsicher sein.
     TTreeComponent = class
     private
       parent, child, left, right: TTreeComponent;
       procedure signout;
       procedure signin(newparent: TTreeComponent);
     public
       function getParent: TTreeComponent;
       constructor create(aowner: TTreeComponent); virtual; reintroduce;
       destructor destroy; override;
       procedure SetParentComponent(newOwner: TTreeComponent);
       procedure SetChildsParentComponent(newOwner: TTreeComponent);
     end;

function ParseFloat(s:string;el:integer):single;

function LeftString(s, pattern: string): string;
function RightString(s, pattern: string): string;

function getPlatformSign: string;

function escapeString(s: string): string;
function unescapeString(s: string): string;

implementation
uses math;

function LeftString(s, pattern: string): string;
begin
 if pos(pattern,s)=0 then result:=s //Alles ist links
 else result:=copy(s, 1, pos(pattern,s)-1);
end;
function RightString(s, pattern: string): string;
begin
 if pos(pattern,s)=0
  then result:='' //nichts ist rechts
  else result:=copy(s, pos(pattern,s)+length(pattern), 12313); //Den String rechts davon extrahieren
end;

function getPlatformSign: string;
begin
  result:={$ifdef windows}'win'{$else}'linux'{$endif}+
          {$ifdef cpu32}'32'{$else}'64'{$endif};
end;

function escapeString(s: string): string;
var i: integer;
begin
 result:='';
 for i:=1 to length(s) do if s[i] in bst then result+=s[i] else result+='$'+IntToHex(byte(s[i]),2);
end;

function unescapeString(s: string): string;
var i: integer;
begin
 result:='';
 i:=1;
 while i<=length(s) do begin
  if s[i] in bst then result+=s[i] else begin
   result+=char(strtointdef(copy(s,i,3),32)); inc(i,2);
  end;
  inc(i);
 end;
end;



function ParseFloat(s:string;el:integer):single;
var i:integer;
begin
 //result:=StrToFloatDef(s, el);
 s:=trim(s);
 if s='' then result:=el else begin
  if s[1]='-' then result:=-ParseFloat(copy(s,2,1231231), el) else begin
   i:=pos('.',s);
   if i=0 then result:=strtoint64def(s,el) else begin
    delete(s,i,1);
    result:=strtoint64def(s,el)*power(0.1,length(s)-i+1);
   end;
  end;
 end;
 if IsNan(Result) then result:=el;
end;


{ TTreeComponent }

procedure TTreeComponent.signout;
begin
 // ToDo: Threadsicher machen
 if parent<>nil then begin
   if parent.child=self then parent.child:=right; // ich war sein erstes Kind..
   if left<>nil then left.right:=right; // aus der LL-List austragen
   if right<>nil then right.left:=left;
   parent:=nil;
   left:=nil;
   right:=nil;
 end;
end;

procedure TTreeComponent.signin(newparent: TTreeComponent);
begin
 parent:=newparent;
 // ToDo: Threadsicher machen (parent locken)
 if parent.child=nil then begin
  parent.child:=self; // das erste Kind
 end else begin
  left:=parent.child;
  right:=left.right;
  left.right:=self;  // an 2. stelle reinquetschen
  if right<>nil then right.left:=self;
 end;
end;

function TTreeComponent.getParent: TTreeComponent;
begin
  result:=parent;
end;

constructor TTreeComponent.create(aowner: TTreeComponent);
begin
  if aowner<>nil then begin
   signin(aowner);
  end;
end;

destructor TTreeComponent.destroy;
var t, t2: TTreeComponent;
begin
  t:=child; // Assert: child.left=nil
  while t<>nil do begin
    t.parent:=nil; // erst mal vergisst er mich..
    t:=t.right;
  end;
  t:=child;
  while t<>nil do begin
    t2:=t;
    t:=t2.right;
    t2.free; // Kinder befreien
  end;
  signout;
  inherited destroy;
end;

procedure TTreeComponent.SetParentComponent(newOwner: TTreeComponent);
begin
 signout;
 signin(newOwner);
end;

procedure TTreeComponent.SetChildsParentComponent(newOwner: TTreeComponent);
begin
  while child<>nil do child.SetParentComponent(newOwner);
end;

end.

